	list p=12F683, b=8, c= 102, n=71, t=on, st=off, f=inhx32
;******************************************************************
;*								  *
;*  Filename: Night Rider 12F683.asm				  *
;*    Author: Mike McLaren, K8LH   (k8lh_at_arrl.net)		  *
;*      Date: 15-Jan-06  (last revision 15-Jan-06)		  *
;*								  *
;*								  *
;*     MPLab: 7.30    (tabs=8)					  *
;*     MPAsm: 5.01						  *
;*								  *
;******************************************************************

	#include	<p12f683.inc>
	errorlevel	-302
;
	__config	_FCMEN_OFF & _IESO_OFF & _MCLRE_OFF & _PWRTE_ON & _WDT_OFF & _INTOSCIO
;
;
LEDBUF	equ	h'30'		; 4-byte buffer [30..33]
LEDPTR	equ	h'34'		; (ISR)
COLPOS	equ	h'35'		; (ISR)
LED_LO	equ	h'36'		;
LED_HI	equ	h'37'		;
LEDPROC	equ	h'38'		; LED process latch
CTRL	equ	h'39'		;
CTRH	equ	h'3A'		;
;
;  file locations used by ISR to save and restore context
;
W_ISR	equ	h'70'		; ISR 'W'
S_ISR	equ	h'71'		; ISR 'STATUS'
P_ISR	equ	h'72'		; ISR 'PCLATH'
F_ISR	equ	h'73'		; ISR 'FSR'
;

;******************************************************************
;*                                                             	  *
;*                                                                *
;*                                                                *
;*                                                                *
;*                                                                *
;******************************************************************

	org	0x0000

RESET	clrf	STATUS		;				  |B0
	clrf	GPIO		;				  |B0
	goto	MAIN		;				  |B0

;******************************************************************
;*                                                             	  *
;*  Interrupt Service Routine					  *
;*                                                                *
;******************************************************************

	org	0x0004
;
;  save main program context
;
ISR	movwf	W_ISR		; save W-reg			  |B?
	swapf	STATUS,W	; doesn't change STATUS bits	  |B?
	movwf	S_ISR		; save STATUS reg		  |B?
	clrf	STATUS		; bank 0			  |B0
	movf	PCLATH,W	;				  |B0
	movwf	P_ISR		; save PCLATH			  |B0
	movf	FSR,W		;				  |B0
	movwf	F_ISR		; save FSR			  |B0
	bcf	PIR1,TMR2IF	; clear TMR2 interrupt flag	  |B0
;
;  multiplex 'scan' code for the LED matrix
;
ISR_LED	bsf	STATUS,RP0	; bank 1			  |B1
	movlw	b'00011111'	;				  |B1
	movwf	TRISIO		; turn off display		  |B1
	bcf	STATUS,RP0	; bank 0			  |B0
	movf	LEDPTR,W	; get LED buffer pointer	  |B0
	movwf	FSR		; setup indirect address	  |B0
	movf	COLPOS,W	; get column bit		  |B0
	movwf	GPIO		; set new output pattern	  |B0
	andwf	INDF,W		; compare to data bit		  |B0
	skpz			; need 'float' bit?		  |B0
	iorlw	b'00010000'	; yes, set GP4 'float' bit	  |B0
	iorwf	COLPOS,W	; pick up column bit		  |B0
	iorwf	INDF,W		; pick up data bits		  |B0
	xorlw	b'00111111'	; invert all bits		  |B0
	bsf	STATUS,RP0	; bank 1			  |B1
	movwf	TRISIO		;				  |B1
	bcf	STATUS,RP0	; bank 0			  |B0
;
;  increment column and prep for next ISR cycle
;
	incf	LEDPTR,f	; increment LED buffer pointer	  |B0
	bcf	STATUS,C	; clear Carry			  |B0
ISR_COL	rlf	COLPOS,f	;				  |B0
	movf	COLPOS,W	;				  |B0
	xorlw	b'00001000'	; bit 3 (RXPIN)?		  |B0
	bz	ISR_COL		; yes, branch			  |B0
	xorlw	b'00101000'	;				  |B0
	skpz			; all 4 columns scanned?	  |B0
	goto	ISR_XIT		; yes, branch			  |B0
	movlw	b'00000001'	;				  |B0
	movwf	COLPOS		; reset COLPOS to column 1	  |B0
	movlw	LEDBUF		;				  |B0
	movwf	LEDPTR		; reset LED buffer pointer	  |B0
;
;  setup ISR scan buffers LEDBUF[0..3] (3 bits/location) from
;  main programs 12-bit LED_LO + LED_HI variables
;
;  [00001111] [11111111] for LEDs 01 through 12
;    LED_HI     LED_LO
;
	movf	LED_LO,W	;				  |B0
	movwf	LEDBUF+1	;				  |B0
	andlw	b'00000111'	;				  |B0
	movwf	LEDBUF+0	; LEDs 01-03			  |B0
	rlf	LEDBUF+1,f	;				  |B0
	swapf	LEDBUF+1,W	;				  |B0
	andlw	b'00000111'	;				  |B0
	movwf	LEDBUF+1	; LEDs 04-06			  |B0
	rrf	LED_HI,W	; -----XXX			  |B0
	andlw	b'00000111'	;				  |B0
	movwf	LEDBUF+3	; LEDs 10-12			  |B0
	rrf	LED_LO,W	; XXX-----			  |B0
	movwf	LEDBUF+2	; LEDs 07-09			  |B0
	rrf	LEDBUF+2,f	; -XXX----			  |B0
	swapf	LEDBUF+2,W	; -----XXX			  |B0
	andlw	b'00000111'	;				  |B0
	movwf	LEDBUF+2	;				  |B0
;
ISR_XIT	movf	F_ISR,W		;				  |B0
	movwf	FSR		; restore FSR			  |B0
	movf	P_ISR,W		;				  |B0
	movwf	PCLATH		; restore PCLATH		  |B0
	swapf	S_ISR,W		;				  |B0
	movwf	STATUS		; restore STATUS		  |B?
	swapf	W_ISR,f		; don't screw up STATUS		  |B?
	swapf	W_ISR,W		; restore W-reg			  |B?
	retfie			; return from interrupt		  |B?

;******************************************************************
;*								  *
;*								  *
;*								  *
;*								  *
;*								  *
;******************************************************************

MAIN	movlw	b'00000111'	;				  |B0
	movwf	CMCON0		; comparator off		  |B0
	clrf	GPIO		; all outputs low		  |B0
	bsf	STATUS,RP0	; bank 1			  |B1
	clrf	ANSEL		; digital I/O			  |B1
	clrf	TRISIO		; set GPIO port = all outputs	  |B1
;
;  setup INTOSC for 8-MHz and wait for it to become stable
;
	movlw	b'01110000'	;				  |B1
	movwf	OSCCON		; 8-mhz INTOSC system clock	  |B1
STABLE	btfss	OSCCON,HTS	; oscillator stable?		  |B1
	goto	STABLE		; no, branch			  |B1
	bcf	STATUS,RP0	; bank 0			  |B0
;
;  initialize LED variables
;
	movlw	b'00000001'	;				  |B0
	movwf	COLPOS		; initialize Column position	  |B0
	movlw	LEDBUF		;				  |B0
	movwf	LEDPTR		; initialize buffer pointer	  |B0
	clrf	LEDBUF+0	; turn on all 20 LEDs		  |B0
	clrf	LEDBUF+1	;				  |B0
	clrf	LEDBUF+2	;				  |B0
	clrf	LEDBUF+3	;				  |B0
	clrf	LED_HI		;				  |B0
	movlw	b'00000001'	;				  |B0
	movwf	LED_LO		;				  |B0
	clrf	LEDPROC		; clear LED process latch var	  |B0
;
;  initialize Timer 2 interrupts
;
;  configure TIMER2 for 125-usec interrupts (8-MHz clock)
;
;  note: INTCON is 00000000 after any reset
;         T2CON is 00000000 after any reset (pre=1, post=1)
;          PIE1 is 000-0000 after any reset
;          PIR1 is 000-0000 after any reset
;
	bsf	STATUS,RP0	; bank 1			  |B1
	bsf	PIE1,TMR2IE	; enable TMR2 interrupts	  |B1
	movlw	d'250'-1	; number of 500-nsec ticks	  |B1
	movwf	PR2		; for 125-usec interrupts	  |B1
	bcf	STATUS,RP0	; bank 0			  |B0
	bsf	INTCON,GIE	; enable global interrupts	  |B0
	bsf	INTCON,PEIE	; enable peripheral interrupts	  |B0
	bsf	T2CON,TMR2ON	; start TMR2			  |B0
;
;  Night Rider code
;
TEST	movlw	high .20000	;				  |B0
	movwf	CTRH		;				  |B0
	movlw	low  .20000	;				  |B0
	movwf	CTRL		;				  |B0
DLY1	decfsz	CTRL,f		;				  |B0
	goto	DLY1		;				  |B0
	decfsz	CTRH,f		;				  |B0
	goto	DLY1		;				  |B0
LEDTST	btfss	LEDPROC,0	; rolling up?			  |B0
	goto	LEDDEC		; no, branch			  |B0
LEDINC	bcf	STATUS,C	; clear Carry			  |B0
	rlf	LED_LO,f	; rotate left			  |B0
	rlf	LED_HI,f	; rotate left			  |B0
	btfss	LED_HI,2	; done?				  |B0
	goto	TEST		; no, loop again		  |B0
	bcf	LEDPROC,0	; set "roll down"		  |B0
	rrf	LED_HI,f	;				  |B0
	rrf	LED_LO,f	;				  |B0
LEDDEC	bcf	STATUS,C	; clear Carry			  |B0
	rrf	LED_HI,f	; rotate right			  |B0
	rrf	LED_LO,f	; rotate right			  |B0
	bnc	TEST		; no, loop again		  |B0
	bsf	LEDPROC,0	; set "roll up"			  |B0
	rlf	LED_LO,f	;				  |B0
	rlf	LED_HI,f	;				  |B0
	goto	LEDINC		;				  |B0


;
;******************************************************************
	end

